/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2023 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "./getnodes.h"
#include "../utils.h"

#include "aqhome/ipc/endpoint_ipc.h"
#include "aqhome/ipc/nodes/msg_ipc_getdevices_req.h"
#include "aqhome/ipc/nodes/msg_ipc_getdevices_rsp.h"
#include "aqhome/ipc/msg_ipc_result.h"
#include "aqhome/msg/msg_node.h"

#include <gwenhywfar/args.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>

#include <time.h>
#include <unistd.h>


#define I18S(msg) msg
#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



static int _doGetNodes(GWEN_DB_NODE *dbArgs);
static int _sendGetDevices(GWEN_MSG_ENDPOINT *epTcp);



int AQH_Tool_GetNodes(GWEN_DB_NODE *dbGlobalArgs, int argc, char **argv)
{
  GWEN_DB_NODE *dbLocalArgs;
  int rv;
  const GWEN_ARGS args[]= {
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,           /* type */
      "tcpAddress",                 /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "t",                          /* short option */
      "tcpaddress",                 /* long option */
      I18S("Specify TCP address to connect to (defaults to 127.0.0.1)"),
      I18S("Specify TCP address to connect to (defaults to 127.0.0.1)")
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Int,            /* type */
      "tcpPort",                    /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "P",                          /* short option */
      "tcpport",                    /* long option */
      I18S("Specify the TCP port to listen on"),
      I18S("Specify the TCP port to listen on")
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Int,            /* type */
      "timeout",                    /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "T",                          /* short option */
      NULL,                         /* long option */
      I18S("Specify timeout in seconds for PONG response"),
      I18S("Specify timeout in seconds for PONG response")
    },
    {
      GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
      GWEN_ArgsType_Int,             /* type */
      "help",                       /* name */
      0,                            /* minnum */
      0,                            /* maxnum */
      "h",                          /* short option */
      "help",                       /* long option */
      "Show this help screen",      /* short description */
      "Show this help screen"       /* long description */
    }
  };

  dbLocalArgs=GWEN_DB_GetGroup(dbGlobalArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     GWEN_ARGS_MODE_ALLOW_FREEPARAM,
                     args,
                     dbLocalArgs);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  return _doGetNodes(dbLocalArgs);
}



int _doGetNodes(GWEN_DB_NODE *dbArgs)
{
  GWEN_MSG_ENDPOINT *epTcp;
  int rv;
  int timeoutInSeconds;

  epTcp=Utils_SetupNodesClientEndpoint(dbArgs);
  if (epTcp==NULL) {
    DBG_ERROR(NULL, "ERROR creating TCP connection");
    return 2;
  }

  timeoutInSeconds=GWEN_DB_GetIntValue(dbArgs, "timeout", 0, 5);

  rv=Utils_SendAcceptedMsgGroups(epTcp, 0);
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  rv=_sendGetDevices(epTcp);
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  for (;;) {
    GWEN_MSG *msg;
    uint16_t code;

    msg=Utils_WaitForSpecificIpcMessage(epTcp, AQH_MSGTYPE_IPC_NODES_GETDEVICES_RSP, timeoutInSeconds);
    if (msg==NULL) {
      DBG_INFO(NULL, "No GET_DEVICE response received.");
      GWEN_MsgEndpoint_free(epTcp);
      return 2;
    }
    code=GWEN_IpcMsg_GetCode(msg);
    if (code==AQH_MSGTYPE_IPC_NODES_RESULT) {
      fprintf(stdout, "No device list (%d)\n", AQH_ResultIpcMsg_GetResultCode(msg));
      GWEN_Msg_free(msg);
      break;
    }
    else if (code==AQH_MSGTYPE_IPC_NODES_GETDEVICES_RSP) {
      uint16_t fwVersion;
      int64_t ts64;
      GWEN_TIMESTAMP *ts;
      uint8_t flags;

      flags=AQH_GetDevicesResponseIpcMsg_GetFlags(msg);
      fwVersion=AQH_GetDevicesResponseIpcMsg_GetFirmwareVersion(msg);
      ts64=AQH_GetDevicesResponseIpcMsg_GetTimestamp(msg);
      ts=GWEN_Timestamp_fromInt64(ts64); /* TODO: fix timestamp */

      fprintf(stdout, "- node: %02x (uid=%04x, fw type=%02x, fw version=%d.%d, timeStamp: %016lx"
              ", pkg out: %d, pkg in: %d, collisions: %d, busy: %d, crc: %d, io: %d)\n",
              AQH_GetDevicesResponseIpcMsg_GetBusAddress(msg),
              AQH_GetDevicesResponseIpcMsg_GetUid(msg),
              AQH_GetDevicesResponseIpcMsg_GetFirmwareType(msg),
              (fwVersion>>8) & 0xff,
              fwVersion & 0xff,
              (unsigned long int)ts64,
              AQH_GetDevicesResponseIpcMsg_GetPkgOut(msg),
              AQH_GetDevicesResponseIpcMsg_GetPkgIn(msg),
              AQH_GetDevicesResponseIpcMsg_GetCollisions(msg),
              AQH_GetDevicesResponseIpcMsg_GetBusy(msg),
              AQH_GetDevicesResponseIpcMsg_GetCrcErrors(msg),
              AQH_GetDevicesResponseIpcMsg_GetIoErrors(msg));
      GWEN_Timestamp_free(ts);
      GWEN_Msg_free(msg);
      if (flags & AQH_MSGIPC_GETDEVICES_RSP_FLAGS_LAST) {
        DBG_INFO(NULL, "Last");
        break;
      }
    }
    else {
      GWEN_Msg_free(msg);
      return GWEN_ERROR_GENERIC;
    }
  }

  GWEN_MsgEndpoint_free(epTcp);
  return 0;
}



int _sendGetDevices(GWEN_MSG_ENDPOINT *epTcp)
{
  GWEN_MSG *msgOut;

  msgOut=AQH_GetDevicesRequestIpcMsg_new(AQH_MSGTYPE_IPC_NODES_GETDEVICES_REQ);
  if (msgOut==NULL) {
    DBG_ERROR(NULL, "Error creating message");
    return GWEN_ERROR_GENERIC;
  }
  GWEN_MsgEndpoint_AddSendMessage(epTcp, msgOut);
  return 0;
}






